@@ -1,3 +1,8 @@
+{-|
+Complete the definition of the web app begun in App.hs.
+This is always done in two files for (TH?) reasons.
+-}
+
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
@@ -5,8 +10,8 @@
 
 module Hledger.Web.Application
   ( makeApplication
-  , makeFoundation
-  , makeFoundationWith
+  , makeApp
+  , makeAppWith
   ) where
 
 import Data.IORef (newIORef, writeIORef)
@@ -26,9 +31,9 @@ import Hledger.Web.Handler.RegisterR
 import Hledger.Web.Import
 import Hledger.Web.WebOptions (WebOpts(serve_,serve_api_), corsPolicy)
 
--- This line actually creates our YesodDispatch instance. It is the second half
--- of the call to mkYesodData which occurs in Foundation.hs. Please see the
--- comments there for more details.
+-- mkYesodDispatch creates our YesodDispatch instance. 
+-- It complements the mkYesodData call in App.hs,
+-- but must be in a separate file for (TH?) reasons.
 mkYesodDispatch "App" resourcesApp
 
 -- This function allocates resources (such as a database connection pool),
@@ -37,25 +42,29 @@ mkYesodDispatch "App" resourcesApp
 -- migrations handled by Yesod.
 makeApplication :: WebOpts -> Journal -> AppConfig DefaultEnv Extra -> IO Application
 makeApplication opts' j' conf' = do
-    foundation <- makeFoundation conf' opts'
-    writeIORef (appJournal foundation) j'
-    (logWare . (corsPolicy opts')) <$> toWaiApp foundation
+    app <- makeApp conf' opts'
+    writeIORef (appJournal app) j'
+    (logWare . (corsPolicy opts')) <$> toWaiApp app
   where
     logWare | development  = logStdoutDev
             | serve_ opts' || serve_api_ opts' = logStdout
             | otherwise    = id
 
-makeFoundation :: AppConfig DefaultEnv Extra -> WebOpts -> IO App
-makeFoundation conf opts' = do
-    manager <- newManager defaultManagerSettings
-    s <- staticSite
-    jref <- newIORef nulljournal
-    return $ App conf s manager opts' jref
+makeApp :: AppConfig DefaultEnv Extra -> WebOpts -> IO App
+makeApp = makeAppWith nulljournal
 
--- Make a Foundation with the given Journal as its state.
-makeFoundationWith :: Journal -> AppConfig DefaultEnv Extra -> WebOpts -> IO App
-makeFoundationWith j' conf opts' = do
-    manager <- newManager defaultManagerSettings
-    s <- staticSite
-    jref <- newIORef j'
-    return $ App conf s manager opts' jref
+-- Make an "App" (defined in App.hs), 
+-- with the given Journal as its state
+-- and the given "AppConfig" and "WebOpts" as its configuration.
+makeAppWith :: Journal -> AppConfig DefaultEnv Extra -> WebOpts -> IO App
+makeAppWith j' aconf wopts = do
+  s    <- staticSite
+  m    <- newManager defaultManagerSettings
+  jref <- newIORef j'
+  return App{
+      settings    = aconf
+    , getStatic   = s
+    , httpManager = m
+    , appOpts     = wopts
+    , appJournal  = jref
+    }
